@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # Licensed under the terms of the MIT License
-# (see spyderlib/__init__.py for details)
+# (see spyder/__init__.py for details)
 
 """
 Bootstrapping Spyder
@@ -28,11 +28,15 @@ parser = optparse.OptionParser(
     usage="python bootstrap.py [options] [-- spyder_options]",
     epilog="""\
 Arguments for Spyder's main script are specified after the --
-symbol (example: `python bootstrap.py -- --show-console`).
+symbol (example: `python bootstrap.py -- --hide-console`).
 Type `python bootstrap.py -- --help` to read about Spyder
 options.""")
 parser.add_option('--gui', default=None,
-                  help="GUI toolkit: pyqt (for PyQt4) or pyside (for PySide)")
+                  help="GUI toolkit: pyqt5 (for PyQt5), pyqt (for PyQt4) or "
+                       "pyside (for PySide, deprecated)")
+parser.add_option('--show-console', action='store_true', default=False,
+                  help="(Deprecated) Does nothing, now the default behavior "
+                  "is to show the console")
 parser.add_option('--hide-console', action='store_true',
                   default=False, help="Hide parent console window (Windows only)")
 parser.add_option('--test', dest="test", action='store_true', default=False,
@@ -41,9 +45,13 @@ parser.add_option('--no-apport', action='store_true',
                   default=False, help="Disable Apport exception hook (Ubuntu)")
 parser.add_option('--debug', action='store_true',
                   default=False, help="Run Spyder in debug mode")
+
 options, args = parser.parse_args()
 
-assert options.gui in (None, 'pyqt', 'pyside'), \
+# Store variable to be used in self.restart (restart spyder instance)
+os.environ['SPYDER_BOOTSTRAP_ARGS'] = str(sys.argv[1:])
+
+assert options.gui in (None, 'pyqt5', 'pyqt', 'pyside'), \
        "Invalid GUI toolkit option '%s'" % options.gui
 
 # For testing purposes
@@ -92,35 +100,9 @@ if sys.excepthook != sys.__excepthook__:
 
 # --- Continue
 
-from spyderlib.utils.vcs import get_hg_revision
-print("Revision %s:%s, Branch: %s" % get_hg_revision(DEVPATH))
-
-sys.path.insert(0, DEVPATH)
-print("01. Patched sys.path with %s" % DEVPATH)
-
-EXTPATH = osp.join(DEVPATH, 'external-py' + sys.version[0])
-if osp.isdir(EXTPATH):
-    sys.path.insert(0, EXTPATH)
-    print("                      and %s" % EXTPATH)
-
-
-# Selecting the GUI toolkit: PySide if installed, otherwise PyQt4
-# (Note: PyQt4 is still the officially supported GUI toolkit for Spyder)
-if options.gui is None:
-    try:
-        import PySide  # analysis:ignore
-        print("02. PySide is detected, selecting (experimental)")
-        os.environ['QT_API'] = 'pyside'
-    except:
-        print("02. No PySide detected, using PyQt4 if available")
-else:
-    print ("02. Skipping GUI toolkit detection")
-    os.environ['QT_API'] = options.gui
-
-
 if options.debug:
     # safety check - Spyder config should not be imported at this point
-    if "spyderlib.baseconfig" in sys.modules:
+    if "spyder.config.base" in sys.modules:
         sys.exit("ERROR: Can't enable debug mode - Spyder is already imported")
     print("0x. Switching debug mode on")
     os.environ["SPYDER_DEBUG"] = "True"
@@ -128,9 +110,36 @@ if options.debug:
     # if operation is successful
 
 
+from spyder.utils.vcs import get_git_revision
+print("Revision %s, Branch: %s" % get_git_revision(DEVPATH))
+
+sys.path.insert(0, DEVPATH)
+print("01. Patched sys.path with %s" % DEVPATH)
+
+
+# Selecting the GUI toolkit: PyQt5 if installed, otherwise PySide or PyQt4
+# (Note: PyQt4 is still the officially supported GUI toolkit for Spyder)
+if options.gui is None:
+    try:
+        import PyQt5  # analysis:ignore
+        print("02. PyQt5 is detected, selecting")
+        os.environ['QT_API'] = 'pyqt5'
+    except ImportError:
+        try:
+            import PyQt4  # analysis:ignore
+            print("02. PyQt4 is detected, selecting")
+            os.environ['QT_API'] = 'pyqt'
+        except ImportError:
+            print("02. No PyQt5 or PyQt4 detected, using PySide if available "
+                  "(deprecated)")
+else:
+    print ("02. Skipping GUI toolkit detection")
+    os.environ['QT_API'] = options.gui
+
+
 # Checking versions (among other things, this has the effect of setting the
 # QT_API environment variable if this has not yet been done just above)
-from spyderlib import get_versions
+from spyder import get_versions
 versions = get_versions(reporev=False)
 print("03. Imported Spyder %s" % versions['spyder'])
 print("    [Python %s %dbits, Qt %s, %s %s on %s]" % \
@@ -138,14 +147,26 @@ print("    [Python %s %dbits, Qt %s, %s %s on %s]" % \
        versions['qt_api'], versions['qt_api_ver'], versions['system']))
 
 
+# Check that we have the right qtpy version
+from spyder.utils import programs
+if not programs.is_module_installed('qtpy', '>=1.1.0'):
+    print("")
+    sys.exit("ERROR: Your qtpy version is outdated. Please install qtpy "
+             "1.1.0 or higher to be able to work with Spyder!")
+
+
 # --- Executing Spyder
 
-if not options.hide_console and os.name == 'nt':
-    print("0x. Enforcing parent console (Windows only)")
-    sys.argv.append("--show-console")  # Windows only: show parent console
+if options.show_console:
+    print("(Deprecated) --show console does nothing, now the default behavior "
+          "is to show the console, use --hide-console if you want to hide it")
 
-print("04. Executing spyder.main()")
-from spyderlib import start_app
+if options.hide_console and os.name == 'nt':
+    print("0x. Hiding parent console (Windows only)")
+    sys.argv.append("--hide-console")  # Windows only: show parent console
+
+print("04. Running Spyder")
+from spyder.app import start
 
 time_lapse = time.time()-time_start
 print("Bootstrap completed in " +
@@ -153,4 +174,4 @@ print("Bootstrap completed in " +
     # gmtime() converts float into tuple, but loses milliseconds
     ("%.4f" % time_lapse).split('.')[1])
 
-start_app.main()
+start.main()
