@@ -39,6 +39,11 @@ DEV = os.environ.get('SPYDER_DEV')
 TEST = os.environ.get('SPYDER_TEST')
 
 
+# To do some adjustments for pytest
+# This env var is defined in runtests.py
+PYTEST = os.environ.get('SPYDER_PYTEST')
+
+
 #==============================================================================
 # Debug helpers
 #==============================================================================
@@ -184,9 +189,6 @@ def is_py2exe_or_cx_Freeze():
     return osp.isfile(osp.join(get_module_path('spyder'), osp.pardir))
 
 
-SCIENTIFIC_STARTUP = get_module_source_path('spyder', 'scientific_startup.py')
-
-
 #==============================================================================
 # Image path list
 #==============================================================================
@@ -209,6 +211,7 @@ def get_image_path(name, default="not_found.png"):
         if osp.isfile(full_path):
             return osp.abspath(full_path)
     if default is not None:
+        img_path = osp.join(get_module_path('spyder'), 'images')
         return osp.abspath(osp.join(img_path, default))
 
 
@@ -348,7 +351,7 @@ def get_translation(modname, dirname=None):
                 return to_text_string(y, "utf-8")
         return translate_gettext
     except IOError as _e:  # analysis:ignore
-        #print "Not using translations (%s)" % _e
+        # Not using translations
         def translate_dumb(x):
             if not is_unicode(x):
                 return to_text_string(x, "utf-8")
@@ -374,24 +377,24 @@ def get_supported_types():
     Note:
     If you update this list, don't forget to update doc/variablexplorer.rst
     """
-    from datetime import date
-    editable_types = [int, float, complex, list, dict, tuple, date
+    from datetime import date, timedelta
+    editable_types = [int, float, complex, list, dict, tuple, date, timedelta
                       ] + list(TEXT_TYPES) + list(INT_TYPES)
     try:
         from numpy import ndarray, matrix, generic
         editable_types += [ndarray, matrix, generic]
-    except ImportError:
+    except:
         pass
     try:
         from pandas import DataFrame, Series, DatetimeIndex
         editable_types += [DataFrame, Series, DatetimeIndex]
-    except ImportError:
+    except:
         pass
     picklable_types = editable_types[:]
     try:
         from spyder.pil_patch import Image
         editable_types.append(Image.Image)
-    except ImportError:
+    except:
         pass
     return dict(picklable=picklable_types, editable=editable_types)
 
@@ -427,7 +430,7 @@ def running_in_mac_app():
 SAVED_CONFIG_FILES = ('help', 'onlinehelp', 'path', 'pylint.results',
                       'spyder.ini', 'temp.py', 'temp.spydata', 'template.py',
                       'history.py', 'history_internal.py', 'workingdir',
-                      '.projects', '.spyderproject', '.ropeproject',
+                      '.projects', '.spyproject', '.ropeproject',
                       'monitor.log', 'monitor_debug.log', 'rope.log',
                       'langconfig', 'spyder.lock')
 
