@@ -52,7 +52,7 @@ from spyder.utils.ipython.kernelspec import SpyderKernelSpec
 from spyder.utils.ipython.style import create_qss_style
 from spyder.utils.qthelpers import create_action, MENU_SEPARATOR
 from spyder.utils import icon_manager as ima
-from spyder.utils import encoding, programs, sourcecode
+from spyder.utils import programs, sourcecode
 from spyder.utils.programs import TEMPDIR
 from spyder.utils.misc import get_error_match, remove_backslashes
 from spyder.widgets.findreplace import FindReplace
@@ -301,10 +301,12 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         ask_box = newcb(_("Ask for confirmation before closing"),
                         'ask_before_closing')
         reset_namespace_box = newcb(
-                _("Ask for confirmation before resetting the namespace."),
+                _("Ask for confirmation before removing all user-defined "
+                  "variables"),
                 'show_reset_namespace_warning',
                 tip=_("This option lets you hide the warning message shown\n"
                       "when resetting the namespace from Spyder."))
+        show_time_box = newcb(_("Show elapsed time"), 'show_elapsed_time')
 
         interface_layout = QVBoxLayout()
         interface_layout.addWidget(banner_box)
@@ -312,6 +314,7 @@ class IPythonConsoleConfigPage(PluginConfigPage):
         interface_layout.addWidget(calltips_box)
         interface_layout.addWidget(ask_box)
         interface_layout.addWidget(reset_namespace_box)
+        interface_layout.addWidget(show_time_box)
         interface_group.setLayout(interface_layout)
 
         comp_group = QGroupBox(_("Completion Type"))
@@ -602,7 +605,8 @@ class IPythonConsole(SpyderPluginWidget):
                              "required to create IPython consoles. Please "
                              "make it writable.")
 
-    def __init__(self, parent, testing=False, test_dir=TEMPDIR):
+    def __init__(self, parent, testing=False, test_dir=TEMPDIR,
+                 test_no_stderr=False):
         """Ipython Console constructor."""
         if PYQT5:
             SpyderPluginWidget.__init__(self, parent, main = parent)
@@ -623,8 +627,11 @@ class IPythonConsole(SpyderPluginWidget):
         self.filenames = []
         self.mainwindow_close = False
         self.create_new_client_if_empty = True
+
+        # Attrs for testing
         self.testing = testing
         self.test_dir = test_dir
+        self.test_no_stderr = test_no_stderr
 
         # Initialize plugin
         if not self.testing:
@@ -646,7 +653,7 @@ class IPythonConsole(SpyderPluginWidget):
             # Fixes Issue 561
             self.tabwidget.setDocumentMode(True)
         self.tabwidget.currentChanged.connect(self.refresh_plugin)
-        self.tabwidget.move_data.connect(self.move_tab)
+        self.tabwidget.tabBar().tabMoved.connect(self.move_tab)
         self.tabwidget.tabBar().sig_change_name.connect(
             self.rename_tabs_after_change)
 
@@ -689,6 +696,10 @@ class IPythonConsole(SpyderPluginWidget):
         help_o = CONF.get('help', 'connect/ipython_console')
         color_scheme_n = 'color_scheme_name'
         color_scheme_o = CONF.get('color_schemes', 'selected')
+        show_time_n = 'show_elapsed_time'
+        show_time_o = self.get_option(show_time_n)
+        reset_namespace_n = 'show_reset_namespace_warning'
+        reset_namespace_o = self.get_option(reset_namespace_n)
         for client in self.clients:
             control = client.get_control()
             if font_n in options:
@@ -697,6 +708,11 @@ class IPythonConsole(SpyderPluginWidget):
                 control.set_help_enabled(help_o)
             if color_scheme_n in options:
                 client.set_color_scheme(color_scheme_o)
+            if show_time_n in options:
+                client.show_time_action.setChecked(show_time_o)
+                client.set_elapsed_time_visible(show_time_o)
+            if reset_namespace_n in options:
+                client.reset_warning = reset_namespace_o
 
     def toggle_view(self, checked):
         """Toggle view"""
@@ -746,7 +762,9 @@ class IPythonConsole(SpyderPluginWidget):
             client = self.tabwidget.currentWidget()
             control = client.get_control()
             control.setFocus()
-            widgets = client.get_toolbar_buttons()+[5]
+            buttons = [[b, -7] for b in client.get_toolbar_buttons()]
+            buttons = sum(buttons, [])[:-1]
+            widgets = [client.create_time_label()] + buttons
         else:
             control = None
             widgets = []
@@ -860,6 +878,7 @@ class IPythonConsole(SpyderPluginWidget):
         norm = lambda text: remove_backslashes(to_text_string(text))
 
         # Select client to execute code on it
+        is_new_client = False
         if current_client:
             client = self.get_current_client()
         else:
@@ -867,6 +886,7 @@ class IPythonConsole(SpyderPluginWidget):
             if client is None:
                 self.create_client_for_file(filename)
                 client = self.get_current_client()
+                is_new_client = True
 
         if client is not None:
             # Internal kernels, use runfile
@@ -887,7 +907,19 @@ class IPythonConsole(SpyderPluginWidget):
                 line += "\"%s\"" % to_text_string(filename)
                 if args:
                     line += " %s" % norm(args)
-            self.execute_code(line, current_client, clear_variables)
+            try:
+                if current_client:
+                    self.execute_code(line, current_client, clear_variables)
+                else:
+                    if is_new_client:
+                        client.shellwidget.silent_execute('%clear')
+                    else:
+                        client.shellwidget.execute('%clear')
+                    client.shellwidget.sig_prompt_ready.connect(
+                            lambda: self.execute_code(line, current_client,
+                                                      clear_variables))
+            except AttributeError:
+                pass
             self.visibility_changed(True)
             self.raise_()
         else:
@@ -927,13 +959,15 @@ class IPythonConsole(SpyderPluginWidget):
                 if not current_client:
                     # Clear console and reset namespace for
                     # dedicated clients
-                    sw.silent_execute('%clear')
-                    sw.silent_execute(
-                        'get_ipython().kernel.close_all_mpl_figures()')
+                    # See issue 5748
+                    try:
+                        sw.sig_prompt_ready.disconnect()
+                    except TypeError:
+                        pass
                     sw.reset_namespace(warning=False, silent=True)
                 elif current_client and clear_variables:
                     sw.reset_namespace(warning=False, silent=True)
-                sw.execute(to_text_string(to_text_string(lines)))
+                sw.execute(to_text_string(lines))
             self.activateWindow()
             self.get_current_client().get_control().setFocus()
 
@@ -952,13 +986,17 @@ class IPythonConsole(SpyderPluginWidget):
         client_id = dict(int_id=to_text_string(self.master_clients),
                          str_id='A')
         cf = self._new_connection_file()
+        show_elapsed_time = self.get_option('show_elapsed_time')
+        reset_warning = self.get_option('show_reset_namespace_warning')
         client = ClientWidget(self, id_=client_id,
                               history_filename=get_conf_path('history.py'),
                               config_options=self.config_options(),
                               additional_options=self.additional_options(),
                               interpreter_versions=self.interpreter_versions(),
                               connection_file=cf,
-                              menu_actions=self.menu_actions)
+                              menu_actions=self.menu_actions,
+                              show_elapsed_time=show_elapsed_time,
+                              reset_warning=reset_warning)
         if self.testing:
             client.stderr_dir = self.test_dir
         self.add_tab(client, name=client.get_name(), filename=filename)
@@ -972,20 +1010,24 @@ class IPythonConsole(SpyderPluginWidget):
         # Else we won't be able to create a client
         if not CONF.get('main_interpreter', 'default'):
             pyexec = CONF.get('main_interpreter', 'executable')
-            ipykernel_present = programs.is_module_installed('ipykernel',
-                                                            interpreter=pyexec)
-            if not ipykernel_present:
+            has_ipykernel = programs.is_module_installed('ipykernel',
+                                                         interpreter=pyexec)
+            has_cloudpickle = programs.is_module_installed('cloudpickle',
+                                                           interpreter=pyexec)
+            if not (has_ipykernel and has_cloudpickle):
                 client.show_kernel_error(_("Your Python environment or "
                                      "installation doesn't "
-                                     "have the <tt>ipykernel</tt> module "
-                                     "installed on it. Without this module is "
-                                     "not possible for Spyder to create a "
+                                     "have the <tt>ipykernel</tt> and "
+                                     "<tt>cloudpickle</tt> modules "
+                                     "installed on it. Without these modules "
+                                     "is not possible for Spyder to create a "
                                      "console for you.<br><br>"
-                                     "You can install <tt>ipykernel</tt> by "
-                                     "running in a terminal:<br><br>"
-                                     "<tt>pip install ipykernel</tt><br><br>"
+                                     "You can install them by running "
+                                     "in a system terminal:<br><br>"
+                                     "<tt>pip install ipykernel cloudpickle</tt>"
+                                     "<br><br>"
                                      "or<br><br>"
-                                     "<tt>conda install ipykernel</tt>"))
+                                     "<tt>conda install ipykernel cloudpickle</tt>"))
                 return
 
         self.connect_client_to_kernel(client)
@@ -1007,12 +1049,11 @@ class IPythonConsole(SpyderPluginWidget):
     def connect_client_to_kernel(self, client):
         """Connect a client to its kernel"""
         connection_file = client.connection_file
-        try:
+
+        if self.test_no_stderr:
+            stderr_file = None
+        else:
             stderr_file = client.stderr_file
-        except PermissionError:
-            error_msg = self.permission_error_msg.format(TEMPDIR)
-            client.show_kernel_error(error_msg)
-            return
 
         km, kc = self.create_kernel_manager_and_kernel_client(connection_file,
                                                               stderr_file)
@@ -1232,8 +1273,18 @@ class IPythonConsole(SpyderPluginWidget):
         if index is not None:
             client = self.tabwidget.widget(index)
 
-        # Close client
-        client.stop_button_click_handler()
+        # Needed to handle a RuntimeError. See issue 5568.
+        try:
+            # Close client
+            client.stop_button_click_handler()
+        except RuntimeError:
+            pass
+
+        # Disconnect timer needed to update elapsed time
+        try:
+            client.timer.timeout.disconnect(client.show_time)
+        except (RuntimeError, TypeError):
+            pass
 
         # Check if related clients or kernels are opened
         # and eventually ask before closing them
@@ -1364,6 +1415,17 @@ class IPythonConsole(SpyderPluginWidget):
                 break
         return client
 
+    def set_elapsed_time(self, client):
+        """Set elapsed time for slave clients."""
+        related_clients = self.get_related_clients(client)
+        for cl in related_clients:
+            if cl.timer is not None:
+                client.create_time_label()
+                client.t0 = cl.t0
+                client.timer.timeout.connect(client.show_time)
+                client.timer.start(1000)
+                break
+
     #------ Public API (for kernels) ------------------------------------------
     def ssh_tunnel(self, *args, **kwargs):
         if os.name == 'nt':
@@ -1416,7 +1478,10 @@ class IPythonConsole(SpyderPluginWidget):
         kernel_manager._kernel_spec = kernel_spec
 
         # Save stderr in a file to read it later in case of errors
-        stderr = codecs.open(stderr_file, 'w', encoding='utf-8')
+        if stderr_file is not None:
+            stderr = codecs.open(stderr_file, 'w', encoding='utf-8')
+        else:
+            stderr = None
         kernel_manager.start_kernel(stderr=stderr)
 
         # Kernel client
@@ -1424,7 +1489,7 @@ class IPythonConsole(SpyderPluginWidget):
 
         # Increase time to detect if a kernel is alive
         # See Issue 3444
-        kernel_client.hb_channel.time_to_dead = 6.0
+        kernel_client.hb_channel.time_to_dead = 18.0
 
         return kernel_manager, kernel_client
 
@@ -1489,16 +1554,25 @@ class IPythonConsole(SpyderPluginWidget):
         self.tabwidget.setTabText(index, client.get_name())
 
     def rename_tabs_after_change(self, given_name):
+        """Rename tabs after a change in name."""
         client = self.get_current_client()
 
+        # Prevent renames that want to assign the same name of
+        # a previous tab
+        repeated = False
+        for cl in self.get_clients():
+            if id(client) != id(cl) and given_name == cl.given_name:
+                repeated = True
+                break
+
         # Rename current client tab to add str_id
-        if client.allow_rename and not u'/' in given_name:
+        if client.allow_rename and not u'/' in given_name and not repeated:
             self.rename_client_tab(client, given_name)
         else:
             self.rename_client_tab(client, None)
 
         # Rename related clients
-        if client.allow_rename and not u'/' in given_name:
+        if client.allow_rename and not u'/' in given_name and not repeated:
             for cl in self.get_related_clients(client):
                 self.rename_client_tab(cl, given_name)
 
@@ -1545,7 +1619,7 @@ class IPythonConsole(SpyderPluginWidget):
         if not osp.isdir(jupyter_runtime_dir()):
             try:
                 os.makedirs(jupyter_runtime_dir())
-            except PermissionError:
+            except (IOError, OSError):
                 return None
         cf = ''
         while not cf:
@@ -1628,6 +1702,8 @@ class IPythonConsole(SpyderPluginWidget):
                          str_id=chr(slave_ord + 1))
 
         # Creating the client
+        show_elapsed_time = self.get_option('show_elapsed_time')
+        reset_warning = self.get_option('show_reset_namespace_warning')
         client = ClientWidget(self,
                               id_=client_id,
                               given_name=given_name,
@@ -1639,7 +1715,9 @@ class IPythonConsole(SpyderPluginWidget):
                               menu_actions=self.menu_actions,
                               hostname=hostname,
                               external_kernel=external_kernel,
-                              slave=True)
+                              slave=True,
+                              show_elapsed_time=show_elapsed_time,
+                              reset_warning=reset_warning)
 
         # Create kernel client
         kernel_client = QtKernelClient(connection_file=connection_file)
@@ -1672,6 +1750,10 @@ class IPythonConsole(SpyderPluginWidget):
                     self.connect_external_kernel)
             client.shellwidget.is_spyder_kernel()
 
+        # Set elapsed time, if possible
+        if not external_kernel:
+            self.set_elapsed_time(client)
+
         # Adding a new tab for the client
         self.add_tab(client, name=client.get_name())
 
