@@ -22,9 +22,10 @@ from qtpy.QtWidgets import QMenu, QMessageBox
 # Local imports
 from spyder.config.base import _, get_home_dir
 from spyder.plugins import SpyderPluginMixin
-from spyder.py3compat import is_text_string, getcwd
+from spyder.py3compat import is_text_string
 from spyder.utils import icon_manager as ima
 from spyder.utils.qthelpers import add_actions, create_action, MENU_SEPARATOR
+from spyder.utils.misc import getcwd_or_home
 from spyder.widgets.projects.explorer import ProjectExplorerWidget
 from spyder.widgets.projects.projectdialog import ProjectDialog
 from spyder.widgets.projects import EmptyProject
@@ -35,7 +36,6 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
 
     CONF_SECTION = 'project_explorer'
 
-    open_terminal = Signal(str)
     open_interpreter = Signal(str)
     pythonpath_changed = Signal()
 
@@ -46,11 +46,13 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
     removed_tree = Signal(str)
     renamed = Signal(str, str)
     redirect_stdio = Signal(bool)
+    run = Signal(str)
 
     # Project handling
     sig_project_created = Signal(object, object, object)
     sig_project_loaded = Signal(object)
     sig_project_closed = Signal(object)
+    sig_new_file = Signal(str)
 
     def __init__(self, parent=None):
         ProjectExplorerWidget.__init__(self, parent=parent,
@@ -136,6 +138,8 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
 
         self.sig_open_file.connect(self.main.open_file)
 
+        self.sig_new_file.connect(lambda x: self.editor.new(text=x))
+
         # New project connections. Order matters!
         self.sig_project_loaded.connect(
             lambda v: self.workingdirectory.chdir(v))
@@ -251,7 +255,7 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
         if self.current_active_project is None:
             if save_previous_files:
                 self.editor.save_open_files()
-            self.editor.set_option('last_working_dir', getcwd())
+            self.editor.set_option('last_working_dir', getcwd_or_home())
             self.show_explorer()
         else: # we are switching projects
             self.set_project_filenames(self.editor.get_open_filenames())
@@ -341,7 +345,8 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
 
     def get_last_working_dir(self):
         """Get the path of the last working directory"""
-        return self.editor.get_option('last_working_dir', default=getcwd())
+        return self.editor.get_option('last_working_dir',
+                                      default=getcwd_or_home())
 
     def save_config(self):
         """Save configuration: opened projects & tree widget state"""
@@ -380,9 +385,7 @@ class Projects(ProjectExplorerWidget, SpyderPluginMixin):
 
     def restart_consoles(self):
         """Restart consoles when closing, opening and switching projects"""
-        self.main.extconsole.restart()
-        if self.main.ipyconsole:
-            self.main.ipyconsole.restart()
+        self.main.ipyconsole.restart()
 
     def is_valid_project(self, path):
         """Check if a directory is a valid Spyder project"""
