@@ -17,8 +17,10 @@ http://docs.python.org/library/profile.html
 from __future__ import with_statement
 import os
 import os.path as osp
+from itertools import islice
 import sys
 import time
+import re
 
 # Third party imports
 from qtpy.compat import getopenfilename, getsavefilename
@@ -29,14 +31,14 @@ from qtpy.QtWidgets import (QApplication, QHBoxLayout, QLabel, QMessageBox,
                             QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget)
 
 # Local imports
-from spyder.config.base import get_conf_path, get_translation
-from spyder.py3compat import getcwd, to_text_string
+from spyder.config.base import get_conf_path, get_translation, debug_print
+from spyder.py3compat import to_text_string
 from spyder.utils import icon_manager as ima
 from spyder.utils.qthelpers import (create_toolbutton, get_item_user_text,
                                     set_item_user_text)
 from spyder.utils.programs import shell_split
 from spyder.widgets.comboboxes import PythonModulesComboBox
-from spyder.utils.misc import add_pathlist_to_PYTHONPATH
+from spyder.utils.misc import add_pathlist_to_PYTHONPATH, getcwd_or_home
 from spyder.widgets.variableexplorer.texteditor import TextEditor
 
 # This is needed for testing this module as a stand alone script
@@ -177,15 +179,16 @@ class ProfilerWidget(QWidget):
     def save_data(self):
         """Save data"""
         title = _( "Save profiler result")
-        filename, _selfilter = getsavefilename(self, title,
-                                               getcwd(),
-                                               _("Profiler result")+" (*.Result)")
+        filename, _selfilter = getsavefilename(
+                self, title, getcwd_or_home(),
+                _("Profiler result")+" (*.Result)")
         if filename:
             self.datatree.save_data(filename)
             
     def compare(self):
-        filename, _selfilter = getopenfilename(self, _("Select script to compare"),
-                                               getcwd(), _("Profiler result")+" (*.Result)")
+        filename, _selfilter = getopenfilename(
+                self, _("Select script to compare"),
+                getcwd_or_home(), _("Profiler result")+" (*.Result)")
         if filename:
             self.datatree.compare(filename)
             self.show_data()
@@ -216,8 +219,9 @@ class ProfilerWidget(QWidget):
             
     def select_file(self):
         self.redirect_stdio.emit(False)
-        filename, _selfilter = getopenfilename(self, _("Select Python script"),
-                           getcwd(), _("Python scripts")+" (*.py ; *.pyw)")
+        filename, _selfilter = getopenfilename(
+                self, _("Select Python script"),
+                getcwd_or_home(), _("Python scripts")+" (*.py ; *.pyw)")
         self.redirect_stdio.emit(True)
         if filename:
             self.analyze(filename)
@@ -351,6 +355,30 @@ class ProfilerWidget(QWidget):
                                                time.localtime())
         self.datelabel.setText(date_text)
 
+def gettime_s(text):
+    """Parse text and returns a time in seconds
+    
+    The text is of the format 0h : 0.min:0.0s:0 ms:0us:0 ns. 
+    Spaces are not taken into account and any of the specifiers can be ignored"""
+    pattern = '([+-]?\d+\.?\d*) ?([munsecinh]+)'
+    matches = re.findall(pattern, text)
+    if len(matches) == 0:
+        return None
+    time = 0.
+    for res in matches:
+        tmp = float(res[0])
+        if res[1] == 'ns':
+            tmp *= 1e-9
+        elif res[1] == 'us':
+            tmp *= 1e-6
+        elif res[1] == 'ms':
+            tmp *= 1e-3
+        elif res[1] == 'min':
+            tmp *= 60
+        elif res[1] == 'h':
+            tmp *= 3600
+        time += tmp
+    return time
 
 class TreeWidgetItem( QTreeWidgetItem ):
     def __init__(self, parent=None):
@@ -359,11 +387,16 @@ class TreeWidgetItem( QTreeWidgetItem ):
     def __lt__(self, otherItem):
         column = self.treeWidget().sortColumn()
         try:
+            if column == 1 or column == 3: #TODO: Hardcoded Column
+                t0 = gettime_s(self.text(column))
+                t1 = gettime_s(otherItem.text(column))
+                if t0 is not None and t1 is not None:
+                    return t0 > t1
+            
             return float( self.text(column) ) > float( otherItem.text(column) )
         except ValueError:
             return self.text(column) > otherItem.text(column)
 
-
 class ProfilerDataTree(QTreeWidget):
     """
     Convenience tree widget (with built-in model) 
@@ -430,7 +463,14 @@ class ProfilerDataTree(QTreeWidget):
         self.profdata = stats_indi[0]
         
         if self.compare_file is not None:
-            stats_indi.append(pstats.Stats(self.compare_file))
+            try:
+                stats_indi.append(pstats.Stats(self.compare_file))
+            except IOError as e:
+                QMessageBox.critical(
+                    self, _("Error"),
+                    _("Error when trying to load profiler results"))
+                debug_print("Error when calling pstats, {}".format(e))
+                self.compare_file = None
         map(lambda x: x.calc_callees(), stats_indi)
         self.profdata.calc_callees()
         self.stats1 = stats_indi
@@ -495,28 +535,76 @@ class ProfilerDataTree(QTreeWidget):
                 node_type = 'constructor'                
             file_and_line = '%s : %d' % (filename, line_number)
         return filename, line_number, function_name, file_and_line, node_type
-        
-    def color_string(self, args):
-        x, format = args
+
+    @staticmethod
+    def format_measure(measure):
+        """Get format and units for data coming from profiler task."""
+        # Convert to a positive value.
+        measure = abs(measure)
+
+        # For number of calls
+        if isinstance(measure, int):
+            return to_text_string(measure)
+
+        # For time measurements
+        if 1.e-9 < measure <= 1.e-6:
+            measure = u"{0:.2f} ns".format(measure / 1.e-9)
+        elif 1.e-6 < measure <= 1.e-3:
+            measure = u"{0:.2f} us".format(measure / 1.e-6)
+        elif 1.e-3 < measure <= 1:
+            measure = u"{0:.2f} ms".format(measure / 1.e-3)
+        elif 1 < measure <= 60:
+            measure = u"{0:.2f} sec".format(measure)
+        elif 60 < measure <= 3600:
+            m, s = divmod(measure, 3600)
+            if s > 60:
+                m, s = divmod(measure, 60)
+                s = to_text_string(s).split(".")[-1]
+            measure = u"{0:.0f}.{1:.2s} min".format(m, s)
+        else:
+            h, m = divmod(measure, 3600)
+            if m > 60:
+                m /= 60
+            measure = u"{0:.0f}h:{1:.0f}min".format(h, m)
+        return measure
+
+    def color_string(self, x):
+        """Return a string formatted delta for the values in x.
+
+        Args:
+            x: 2-item list of integers (representing number of calls) or
+               2-item list of floats (representing seconds of runtime).
+
+        Returns:
+            A list with [formatted x[0], [color, formatted delta]], where
+            color reflects whether x[1] is lower, greater, or the same as
+            x[0].
+        """
         diff_str = ""
         color = "black"
+
         if len(x) == 2 and self.compare_file is not None:
             difference = x[0] - x[1]
-            if difference < 0:
-                diff_str = "".join(['',format[1] % difference])
-                color = "green"
-            elif difference > 0:
-                diff_str = "".join(['+',format[1] % difference])
-                color = "red"
-        return [format[0] % x[0], [diff_str, color]]
+            if difference:
+                color, sign = ('green', '-') if difference < 0 else ('red', '+')
+                diff_str = '{}{}'.format(sign, self.format_measure(difference))
+        return [self.format_measure(x[0]), [diff_str, color]]
 
+    def format_output(self, child_key):
+        """ Formats the data.
+
+        self.stats1 contains a list of one or two pstat.Stats() instances, with
+        the first being the current run and the second, the saved run, if it
+        exists.  Each Stats instance is a dictionary mapping a function to
+        5 data points - cumulative calls, number of calls, total time,
+        cumulative time, and callers.
+
+        format_output() converts the number of calls, total time, and
+        cumulative time to a string format for the child_key parameter.
+        """
+        data = [x.stats.get(child_key, [0, 0, 0, 0, {}]) for x in self.stats1]
+        return (map(self.color_string, islice(zip(*data), 1, 4)))
 
-    def format_output(self,child_key):
-        """ Formats the data"""
-        if True:
-            data = [x.stats.get(child_key,[0,0,0,0,0]) for x in self.stats1]
-            return map(self.color_string,zip(list(zip(*data))[1:4], [["%i"]*2, ["%.3f","%.3f"], ["%.3f","%.3f"]]))
-            
     def populate_tree(self, parentItem, children_list):
         """Recursive method to create each item (and associated data) in the tree."""
         for child_key in children_list:
@@ -527,8 +615,6 @@ class ProfilerDataTree(QTreeWidget):
             ((total_calls, total_calls_dif), (loc_time, loc_time_dif), (cum_time,
              cum_time_dif)) = self.format_output(child_key)
 
-            (primcalls, total_calls, loc_time, cum_time, callers
-             ) = self.stats[child_key]
             child_item = TreeWidgetItem(parentItem)
             self.item_list.append(child_item)
             self.set_item_data(child_item, filename, line_number)
@@ -599,8 +685,8 @@ class ProfilerDataTree(QTreeWidget):
         while ancestor:
             if (child_item.data(0, Qt.DisplayRole
                                 ) == ancestor.data(0, Qt.DisplayRole) and
-                child_item.data(4, Qt.DisplayRole
-                                ) == ancestor.data(4, Qt.DisplayRole)):
+                child_item.data(7, Qt.DisplayRole
+                                ) == ancestor.data(7, Qt.DisplayRole)):
                 return True
             else:
                 ancestor = ancestor.parent()
